// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define AID_ADI 31
#define AID_KEAS 32
#define AID_ROLL 33
#define AID_ALT 34
#define AID_RNG 35
#define AID_GEES 36
#define AID_XPNTR 37
#define AID_AIRDATALEFT 38
#define AID_AIRDATARIGHT 39
#define AID_MAINP 40
#define AID_HOVER 41
#define AID_XPNTRSCALE 42
#define AID_DED 43
#define AID_RADARSWITCH 44
#define AID_RCS 45
#define AID_ADIMODE 46
#define AID_RADIOSWITCH 47
#define AID_MNASWITCH 48
#define AID_MNBSWITCH 49
#define AID_MNACB 50
#define AID_MNBCB 51
#define AID_BAT1CB 52
#define AID_BAT2CB 53
#define AID_IMUSWITCH 54
#define AID_IMUMNA 55
#define AID_IMUMNB 56
#define AID_ADPMNA 57
#define AID_ADPMNB 58
#define AID_RADIOMNA 59
#define AID_RADIOMNB 60
#define AID_RDRMNA 61
#define AID_RDRMNB 62
#define AID_AF 63
#define AID_MAIN 64
#define AID_HVR 65
#include "needles.h"
#include "orbitersdk.h"
#include "OrbiterSoundSDK50.h" //YOU SHOULD PUT THE RUNTIME LIBRARY INTO MULTITHREADED, OTHERWISE ORBITERSOUND WILL NOT WORK

// ==============================================================
// Some vessel parameters
// ==============================================================
const double  PB_SIZE       = 3.5;             // mean radius [m]
const VECTOR3 PB_CS         = {10.5,15.0,5.8}; // x,y,z cross sections [m^2]
const VECTOR3 PB_PMI        = {2.28,2.31,0.79};// principal moments of inertia (mass-normalised) [m^2]
const VECTOR3 PB_RD         = {0.025,0.025,0.02};//{0.05,0.1,0.05};  // rotation drag coefficients
const double  PB_EMPTYMASS  = 500.0;           // empty vessel mass [kg]
const double  PB_FUELMASS   = 750.0;           // max fuel mass [kg]
const double  PB_ISP        = 5e4;             // fuel-specific impulse [m/s]
const VECTOR3 PB_COP        = {0,0,0};//{0,0,-0.1};      // centre of pressure for airfoils [m]
const double  PB_VLIFT_C    = 2.0;             // chord length [m]
const double  PB_VLIFT_S    = 2.0;             // wing area [m^2]
const double  PB_VLIFT_A    = 2.5;             // wing aspect ratio
const double  PB_HLIFT_C    = 2.0;             // chord length [m]
const double  PB_HLIFT_S    = 1.5;             // wing area [m^2]
const double  PB_HLIFT_A    = 2.0;             // wing aspect ratio
MESHHANDLE hPanelMesh = NULL;
const double  PB_MAXMAINTH  = 3e4;             
const double  PB_MAXHOVERTH = 1.5e4;
const double  PB_MAXRCSTH   = 2e2;
HINSTANCE hDll;
const VECTOR3 PB_DOCK_POS   = {0,1.3,-1};      // docking port location [m]
const VECTOR3 PB_DOCK_DIR   = {0,1,0};         // docking port approach direction
const VECTOR3 PB_DOCK_ROT   = {0,0,-1};        // docking port alignment direction
HDC hDC;
DWORD thing, notathing, geez, VSthing, XPNTRthing, barothing, toomuchthing;
//we define some important variables
vesselstate BetterPB;
double t = oapiGetSimTime();
double d = ATMD;
double KEAS = 0;
double vsfps = 0;
double timestp = 1;
double acc = 0;
double fioul = 0;
double vs = 1, vz = 1, vx = 1;
double alt, agl, mach, aoa,slip, volta, voltb, amperea, ampereb;
VECTOR3 weight, tforce, tas, gs;
THRUSTER_HANDLE th_main, th_hover, th_rcs[14], th_group[4];
PROPELLANT_HANDLE hpr;
// Define impact convex hull
static const DWORD ntdvtx = 12;
static TOUCHDOWNVTX tdvtx[ntdvtx] = {
	{_V( 0,  -1.5, 2  ), 2e4, 1e3, 1.6, 1},
	{_V(-1,  -1.5,-1.5), 2e4, 1e3, 3.0, 1},
	{_V( 1,  -1.5,-1.5), 2e4, 1e3, 3.0, 1},
	{_V(-0.5,-0.75,3  ), 2e4, 1e3, 3.0},
	{_V( 0.5,-0.75,3  ), 2e4, 1e3, 3.0},
	{_V(-2.6,-1.1,-1.9), 2e4, 1e3, 3.0},
	{_V( 2.6,-1.1,-1.9), 2e4, 1e3, 3.0},
	{_V(-1,   1.3, 0  ), 2e4, 1e3, 3.0},
	{_V( 1,   1.3, 0  ), 2e4, 1e3, 3.0},
	{_V(-1,   1.3,-2  ), 2e4, 1e3, 3.0},
	{_V( 1,   1.3,-2  ), 2e4, 1e3, 3.0},
	{_V( 0,   0.3,-3.8), 2e4, 1e3, 3.0}
};

// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
double LiftCoeff (double aoa)
{
	int i;
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-60*RAD,-30*RAD,-1*RAD,15*RAD,20*RAD,25*RAD,60*RAD,180*RAD};
	static const double CL[nlift]  = {       0,      0,   -0.1,     0,   0.2,  0.25,   0.2,     0,      0};
	static const double SCL[nlift] = {(CL[1]-CL[0])/(AOA[1]-AOA[0]), (CL[2]-CL[1])/(AOA[2]-AOA[1]),
		                              (CL[3]-CL[2])/(AOA[3]-AOA[2]), (CL[4]-CL[3])/(AOA[4]-AOA[3]),
									  (CL[5]-CL[4])/(AOA[5]-AOA[4]), (CL[6]-CL[5])/(AOA[6]-AOA[5]),
									  (CL[7]-CL[6])/(AOA[7]-AOA[6]), (CL[8]-CL[7])/(AOA[8]-AOA[7])};
	for (i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	return CL[i] + (aoa-AOA[i])*SCL[i];
}
// ==============================================================
// Shuttle-PB class interface
// ==============================================================
class ShuttlePB : public VESSEL3 {
public:
	ShuttlePB(OBJHANDLE hVessel, int flightmodel);
	~ShuttlePB();
	void clbkPostCreation(); //Only used for sound
	void clbkSetClassCaps(FILEHANDLE cfg);
	void clbkLoadStateEx(FILEHANDLE scn, void* vs);
	void clbkSaveState(FILEHANDLE scn);
	//bool clbkDrawHUD(int mode, const HUDPAINTSPEC* hps, oapi::Sketchpad* skp);
	void DefineMainPanel(PANELHANDLE hPanel); //IMPORTANT
	void clbkPostStep(double simt, double simdt, double mjd);
	void ScalePanel(PANELHANDLE hPanel, DWORD viewW, DWORD viewH);
	bool clbkLoadPanel2D(int id, PANELHANDLE hPanel,
		DWORD viewW, DWORD viewH);
	bool clbkPanelRedrawEvent(int id, int event, SURFHANDLE surf); //IMPORTANT
	bool clbkPanelMouseEvent(int id, int event, int mx, int my); //IMPORTANT
	void ADIinit(MESHHANDLE hMesh, DWORD grpidx); //IMPORTANT
	void ADIredraw(double pit, double roll); //IMPORTANT
	void Rollinit(MESHHANDLE hMesh, DWORD grpidx);
	void Ginit(MESHHANDLE hMesh, DWORD grpidx);
	void Gredraw(double gz);
	void VSinit(MESHHANDLE hMesh, DWORD grpidx);
	void VSredraw(double vs);
	void Baroinit(MESHHANDLE hMesh, DWORD grpidx);
	void Baroredraw(double balt);
	void Varioinit(MESHHANDLE hMesh, DWORD grpidx);
	void Varioredraw(double vario);
	void ADLinit(MESHHANDLE hMesh, DWORD grpidx);
	void ADRinit(MESHHANDLE hMesh, DWORD grpidx);
	void ADLredraw(bool on);
	void ADRredraw(bool on);
	double IMUd[3] = { 0.0,0.0,0.0 };
	double RPY[3] = { 0.0,0.0,0.0 };
	//bool clbkDrawHUD(int mode, const HUDPAINTSPEC* hps, oapi::Sketchpad* skp);
	//void Deddraw();
	void truc(int mode);
	int MyID = -1;							// This is your unique Id, it will identify your ship in OrbiterSound.
	void LoadAndPlay(char* WavFilename);	// A MUST HAVE (optional) helper function that 
	// will allow you to play hundreds of event sounds in one call.
	int iSlotNbrLoadAndPlay = 53;		// A variable for the helper function "LoadAndPlay"
	int vtxofs = 0;
	int vtxofs2 = 0;
	int vtxofs3 = 0;
	int vtxofs4 = 0;
	Needle* fuelel,*batt1el, * batt2el, * MNAVel, * MNBVel, * MNAAel, * MNBAel;
	Electricity* elec;
	MESHGROUP* adigrp = NULL; // mesh group of ADI
	MESHGROUP* geezgrp = NULL; // mesh group of G indicator needle
	MESHGROUP* grproll = NULL; // mesh group of Roll indicator
	MESHGROUP* vsgrp = NULL; // mesh group of RA V/S indicator
	MESHGROUP* barogrp = NULL; // mesh group of Barometric altimeter indicator // NOTE: I USED IT ALSO ON TOTALLY DIFFERENT STUFF
	static SURFHANDLE panel2dtex;
	static SURFHANDLE buttons;
	static SURFHANDLE adisurf;
	static SURFHANDLE instsurf;
	static SURFHANDLE ded;

private:
	static void vlift(VESSEL* v, double aoa, double M, double Re,
		void* context, double* cl, double* cm, double* cd);
	static void hlift(VESSEL* v, double aoa, double M, double Re,
		void* context, double* cl, double* cm, double* cd);

	// transformations for control surface animations
	static MGROUP_ROTATE trans_Laileron, trans_Raileron;
	static MGROUP_ROTATE trans_Lelevator, trans_Relevator;
};
SURFHANDLE ShuttlePB::panel2dtex = NULL;
SURFHANDLE ShuttlePB::buttons = NULL;
SURFHANDLE ShuttlePB::adisurf = NULL;
SURFHANDLE ShuttlePB::instsurf = NULL;
SURFHANDLE ShuttlePB::ded = NULL;
ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
	hPanelMesh = NULL;
	fuelel = new Needle(856, 521);
	batt1el = new Needle(1074, 683);
	batt2el = new Needle(1074, 833);
	MNAVel = new Needle(1230, 506);
	MNBVel = new Needle(1381, 506);
	MNAAel = new Needle(1532, 506);
	MNBAel = new Needle(1683, 506);
	elec = new Electricity(this);
}

ShuttlePB::~ShuttlePB ()
{
	if (hPanelMesh) oapiDeleteMesh(hPanelMesh);
}

void ShuttlePB::clbkPostCreation() {
	MyID = ConnectToOrbiterSoundDLL(GetHandle());
}

// animation transformation definitions
static UINT GRP_LWING = 2;
static UINT GRP_RWING = 3;
static VECTOR3 LWING_REF  = {-1.3,-0.725,-1.5};
static VECTOR3 LWING_AXIS = {-0.9619,-0.2735,0};
static VECTOR3 RWING_REF  = {1.3,-0.725,-1.5};
static VECTOR3 RWING_AXIS = {0.9619,-0.2735,0};
static float AILERON_RANGE = (float)(20.0*RAD);
static float ELEVATOR_RANGE = (float)(30.0*RAD);
MGROUP_ROTATE ShuttlePB::trans_Laileron (0, &GRP_LWING, 1, LWING_REF, LWING_AXIS, AILERON_RANGE);
MGROUP_ROTATE ShuttlePB::trans_Raileron (0, &GRP_RWING, 1, RWING_REF, RWING_AXIS, AILERON_RANGE);
MGROUP_ROTATE ShuttlePB::trans_Lelevator (0, &GRP_LWING, 1, LWING_REF, LWING_AXIS, -ELEVATOR_RANGE);
MGROUP_ROTATE ShuttlePB::trans_Relevator (0, &GRP_RWING, 1, RWING_REF, RWING_AXIS, ELEVATOR_RANGE);

void ShuttlePB::LoadAndPlay(char* WavFilename) { //YOU SHOULD USE A WAV FILE, OTHERWISE IT'LL NOT WORK!!!!!
	iSlotNbrLoadAndPlay++;  // will use the next slot
	// Ensure that we use only our three slot (no need to initialise this value, it will be done at the first call)
	if (iSlotNbrLoadAndPlay > 55 || iSlotNbrLoadAndPlay)           iSlotNbrLoadAndPlay = 53;

		char FilenamePath[255] = { 0 };
		//sprintf(FilenamePath, "Sound\\_CustomVesselsSounds\\BetterPB\\%s", WavFilename);
		RequestLoadVesselWave(MyID, iSlotNbrLoadAndPlay, WavFilename, INTERNAL_ONLY);
		PlayVesselWave(MyID, iSlotNbrLoadAndPlay);
}
void ShuttlePB::truc(int mode) {
	VECTOR3 axis1, axis2, axis3;
	OBJHANDLE hRef;
	switch (mode) {// reference axis
	case -1:
		axis1 = _V(1, 0, 0);
		axis2 = _V(0, 1, 0);
		axis3 = _V(0, 0, 1);
		break;
	case 0:
		hRef = GetGravityRef();
		GetRelativeVel(hRef, axis3);
		axis3 = unit(axis3);
		VECTOR3 vv;
		GetRelativePos(hRef, vv);    // local vertical (y)
		axis1 = unit(crossp(vv,axis3));    // direction of orbital momentum (x)
		axis2 = unit(crossp(axis3,axis1));
		break;
	case 1:
		hRef = GetSurfaceRef();
		GetRelativePos(hRef, axis2);
		axis2 = unit(axis2);
		MATRIX3 prot;
		oapiGetRotationMatrix(hRef, &prot);
		VECTOR3 paxis = { prot.m12, prot.m22, prot.m32 };  // planet rotation axis in global frame
		VECTOR3 yaxis = unit(crossp(paxis, axis2));      // direction of yaw=+90 pole in global frame
		axis3 = crossp(axis2, yaxis);
		axis1 = crossp(axis2, axis3);
		break;
	}
	MATRIX3 R = _M(axis1.x, axis2.x, axis3.x, axis1.y, axis2.y, axis3.y, axis1.z, axis2.z, axis3.z);
	MATRIX3 srot;
	GetRotationMatrix(srot);

	VECTOR3 shipx = { srot.m11, srot.m21, srot.m31 };
	VECTOR3 shipy = { srot.m12, srot.m22, srot.m32 };
	VECTOR3 shipz = { srot.m13, srot.m23, srot.m33 };
	shipx = tmul(R, shipx);
	shipy = tmul(R, shipy);
	shipz = tmul(R, shipz);


	RPY[0] = atan2(shipx.y, shipy.y);  
	RPY[1] = asin(shipz.y);            
	RPY[2] = atan2(shipz.x, shipz.z);
}
//void ShuttlePB::Landingautopilot(double rollr, double pitchr, double yawr) {}
// TEXTURING OFFSET (1024 - 540)
int off = 484;
// ==============================================================
// Overloaded callback functions
// ==============================================================
// first, can our module at least remember a thing? We will save our internal state !
void ShuttlePB::clbkSaveState(FILEHANDLE scn)
{
	char cbuf[256];
	// Write default and subsystem vessel parameters
	VESSEL3::clbkSaveState(scn);

	// Write custom parameters
	// You can enter a bool as an int, it will work.
	// SWITCHES
	oapiWriteScenario_int(scn, "XPNDRSCALE", BetterPB.scale);
	oapiWriteScenario_int(scn, "ADR", BetterPB.ADRext_p);
	oapiWriteScenario_int(scn, "ADL", BetterPB.ADLext_p);
	oapiWriteScenario_int(scn, "RADAR", BetterPB.radar);
	oapiWriteScenario_float(scn, "BAT1L", BetterPB.batt1);
	oapiWriteScenario_float(scn, "BAT2L", BetterPB.batt2);
	oapiWriteScenario_int(scn, "RADIO", BetterPB.radio);
	oapiWriteScenario_int(scn, "B1B2A", BetterPB.MNA_b1b2);
	oapiWriteScenario_int(scn, "B1B2B", BetterPB.MNB_b1b2);
	oapiWriteScenario_int(scn, "IMU", BetterPB.IMU_p);
	oapiWriteScenario_int(scn, "BAT1", BetterPB.BATT1);
	oapiWriteScenario_int(scn, "BAT2", BetterPB.BATT2);
	oapiWriteScenario_int(scn, "MNA", BetterPB.MNA);
	oapiWriteScenario_int(scn, "MNB", BetterPB.MNB);
	oapiWriteScenario_int(scn, "IMUMNA", BetterPB.IMU_MNA);
	oapiWriteScenario_int(scn, "IMUMNB", BetterPB.IMU_MNB);
	oapiWriteScenario_int(scn, "ADPMNA", BetterPB.ADP_MNA);
	oapiWriteScenario_int(scn, "ADPMNB", BetterPB.ADP_MNB);
	oapiWriteScenario_int(scn, "RADMNA", BetterPB.RAD_MNA);
	oapiWriteScenario_int(scn, "RADMNB", BetterPB.RAD_MNB);
	oapiWriteScenario_int(scn, "RDRMNA", BetterPB.RDR_MNA);
	oapiWriteScenario_int(scn, "RDRMNB", BetterPB.RDR_MNB);
	oapiWriteScenario_int(scn, "RCS", BetterPB.rcs);
	oapiWriteScenario_int(scn, "AFCTRL", BetterPB.af);
	oapiWriteScenario_int(scn, "ADI", BetterPB.adimode);
	oapiWriteScenario_int(scn, "HVR", BetterPB.hvr_arm);
	oapiWriteScenario_int(scn, "MAIN", BetterPB.main_arm);
	oapiWriteScenario_int(scn, "BAT1", BetterPB.BATT1);
	oapiWriteScenario_int(scn, "BAT2", BetterPB.BATT2);
}
void ShuttlePB::clbkLoadStateEx(FILEHANDLE scn, void* vs)
{
	char* line;
	int buffer;
	float buff;
	while (oapiReadScenario_nextline(scn, line)) {
		if (!_strnicmp(line, "XPNDRSCALE", 10)) { sscanf_s(line + 10, "%d", &(BetterPB.scale)); }
		else if(!_strnicmp(line, "ADL", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.ADLext_p = buffer;
		}
		else if (!_strnicmp(line, "ADR", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.ADRext_p = buffer;
		}
		else if (!_strnicmp(line, "RADAR", 5)) {
			sscanf_s(line + 5, "%d", &buffer);
			BetterPB.radar = buffer;
		}
		else if (!_strnicmp(line, "RADIO", 5)) {
			sscanf_s(line + 5, "%d", &buffer);
			BetterPB.radio = buffer;
		}
		else if (!_strnicmp(line, "IMU", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.IMU_p = buffer;
		}
		else if (!_strnicmp(line, "B1B2A", 5)) {
			sscanf_s(line + 5, "%d", &buffer);
			BetterPB.MNA_b1b2 = buffer;
		}
		else if (!_strnicmp(line, "B1B2B", 5)) {
			sscanf_s(line + 5, "%d", &buffer);
			BetterPB.MNB_b1b2 = buffer;
		}
		else if (!_strnicmp(line, "BAT1L", 5)) {
			sscanf_s(line + 5, "%f", &buff);
			BetterPB.batt1 = buff;
		}
		else if (!_strnicmp(line, "BAT2L", 5)) {
			sscanf_s(line + 5, "%f", &buff);
			BetterPB.batt2 = buff;
		}
		else if (!_strnicmp(line, "MNA", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.MNA = buffer;
		}
		else if (!_strnicmp(line, "MNB", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.MNB = buffer;
		}
		else if (!_strnicmp(line, "IMUMNA", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.IMU_MNA = buffer;
		}
		else if (!_strnicmp(line, "IMUMNB", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.IMU_MNB = buffer;
		}
		else if (!_strnicmp(line, "ADPMNA", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.ADP_MNA = buffer;
		}
		else if (!_strnicmp(line, "ADPMNB", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.ADP_MNB = buffer;
		}
		else if (!_strnicmp(line, "RDRMNA", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.RDR_MNA = buffer;
		}
		else if (!_strnicmp(line, "RDRMNB", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.RDR_MNB = buffer;
		}
		else if (!_strnicmp(line, "RADMNA", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.RAD_MNA = buffer;
		}
		else if (!_strnicmp(line, "RADMNB", 6)) {
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.RAD_MNB = buffer;
		}
		else if (!_strnicmp(line, "ADI", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.adimode = buffer;
		}
		else if (!_strnicmp(line, "RCS", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.rcs = buffer;
		}
		else if (!_strnicmp(line, "AFCTRL", 6)) { // I dont't know why, but when there is only two characters, it fails.
			sscanf_s(line + 6, "%d", &buffer);
			BetterPB.af = buffer;
		}
		else if (!_strnicmp(line, "HVR", 3)) {
			sscanf_s(line + 3, "%d", &buffer);
			BetterPB.hvr_arm = buffer;
		}
		else if (!_strnicmp(line, "MAIN", 4)) {
			sscanf_s(line + 4, "%d", &buffer);
			BetterPB.main_arm = buffer;
		}
		else if (!_strnicmp(line, "BAT1", 4)) {
			sscanf_s(line + 4, "%d", &buffer);
			BetterPB.BATT1 = buffer;
		}
		else if (!_strnicmp(line, "BAT2", 4)) {
			sscanf_s(line + 4, "%d", &buffer);
			BetterPB.BATT2 = buffer;
		}
		else {
			//if (!VESSEL3::clbkParseScenarioLine(line))
				// unrecognised option - pass to Orbiter's generic parser
				ParseScenarioLineEx(line, vs);
		};
	}
}
// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	

	// physical vessel parameters
	SetSize (PB_SIZE);
	SetEmptyMass (PB_EMPTYMASS);
	SetPMI (PB_PMI);
	SetCrossSections (PB_CS);
	SetRotDrag (PB_RD);
	SetTouchdownPoints (tdvtx, ntdvtx);

	// docking port definitions
	SetDockParams (PB_DOCK_POS, PB_DOCK_DIR, PB_DOCK_ROT);

	// airfoil definitions
	CreateAirfoil3 (LIFT_VERTICAL,   PB_COP, vlift, NULL, PB_VLIFT_C, PB_VLIFT_S, PB_VLIFT_A);
	CreateAirfoil3 (LIFT_HORIZONTAL, PB_COP, hlift, NULL, PB_HLIFT_C, PB_HLIFT_S, PB_HLIFT_A);

	// control surface animations
	UINT anim_Laileron = CreateAnimation (0.5);
	UINT anim_Raileron = CreateAnimation (0.5);
	UINT anim_elevator = CreateAnimation (0.5);
	AddAnimationComponent (anim_Laileron, 0, 1, &trans_Laileron);
	AddAnimationComponent (anim_Raileron, 0, 1, &trans_Raileron);
	AddAnimationComponent (anim_elevator, 0, 1, &trans_Lelevator);
	AddAnimationComponent (anim_elevator, 0, 1, &trans_Relevator);

	// aerodynamic control surface defintions
	CreateControlSurface (AIRCTRL_ELEVATOR, 1.5, 0.7, _V( 0,0,-2.5), AIRCTRL_AXIS_XPOS, anim_elevator);
	CreateControlSurface (AIRCTRL_AILERON, 1.5, 0.25, _V( 1,0,-2.5), AIRCTRL_AXIS_XPOS, anim_Laileron);
	CreateControlSurface (AIRCTRL_AILERON, 1.5, 0.25, _V(-1,0,-2.5), AIRCTRL_AXIS_XNEG, anim_Raileron);

	// propellant resources
	hpr = CreatePropellantResource (PB_FUELMASS);
	// main engine
	th_main = CreateThruster (_V(0,0,-4.35), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	AddExhaust (th_main, 8, 1, _V(0,0.3,-4.35), _V(0,0,-1));

	PARTICLESTREAMSPEC contrail_main = {
		0, 5.0, 16, 200, 0.15, 1.0, 5, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	PARTICLESTREAMSPEC exhaust_main = {
		0, 2.0, 20, 200, 0.05, 0.1, 8, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	AddExhaustStream (th_main, _V(0,0.3,-10), &contrail_main);
	AddExhaustStream (th_main, _V(0,0.3,-5), &exhaust_main);

	// hover engine
	th_hover = CreateThruster (_V(0,-1.5,0), _V(0,1,0), PB_MAXHOVERTH, hpr, PB_ISP);
	CreateThrusterGroup (&th_hover, 1, THGROUP_HOVER);
	AddExhaust (th_hover, 8, 1, _V(0,-1.5,1), _V(0,-1,0));
	AddExhaust (th_hover, 8, 1, _V(0,-1.5,-1), _V(0,-1,0));

	PARTICLESTREAMSPEC contrail_hover = {
		0, 5.0, 8, 200, 0.15, 1.0, 5, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	PARTICLESTREAMSPEC exhaust_hover = {
		0, 2.0, 10, 200, 0.05, 0.05, 8, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

	AddExhaustStream (th_hover, _V(0,-3, 1), &contrail_hover);
	AddExhaustStream (th_hover, _V(0,-3,-1), &contrail_hover);
	AddExhaustStream (th_hover, _V(0,-2, 1), &exhaust_hover);
	AddExhaustStream (th_hover, _V(0,-2,-1), &exhaust_hover);

	// RCS engines
	th_rcs[ 0] = CreateThruster (_V( 1,0, 3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 1] = CreateThruster (_V( 1,0, 3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 2] = CreateThruster (_V(-1,0, 3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 3] = CreateThruster (_V(-1,0, 3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 4] = CreateThruster (_V( 1,0,-3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 5] = CreateThruster (_V( 1,0,-3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 6] = CreateThruster (_V(-1,0,-3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 7] = CreateThruster (_V(-1,0,-3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 8] = CreateThruster (_V( 1,0, 3), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 9] = CreateThruster (_V(-1,0, 3), _V( 1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[10] = CreateThruster (_V( 1,0,-3), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[11] = CreateThruster (_V(-1,0,-3), _V( 1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[12] = CreateThruster (_V( 0,0,-3), _V(0,0, 1), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[13] = CreateThruster (_V( 0,0, 3), _V(0,0,-1), PB_MAXRCSTH, hpr, PB_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);

	// camera parameters
	SetCameraOffset (_V(0,0.8,0));

	// associate a mesh for the visual
	AddMesh ("ShuttlePB");
}
void ShuttlePB::clbkPostStep(double simt, double simdt, double mjd) { //It updates all the data for us
	elec->update(BetterPB);
	volta = elec->GetVoltageMNA();
	voltb = elec->GetVoltageMNB();
	amperea = elec->GetAmpereMNA();
	ampereb = elec->GetAmpereMNB();
	if(BetterPB.batt1 > 0) BetterPB.batt1 -= simdt * (elec->GetLoadBAT1())/3600000;
	if (BetterPB.batt2 > 0) BetterPB.batt2 -= simdt * (elec->GetLoadBAT2()) / 3600000;
	GetForceVector(tforce);
	GetWeightVector(weight);
	agl = GetAltitude(ALTMODE_GROUND, NULL) / 0.3048;
	GetAirspeedVector(FRAME_HORIZON, tas);
	aoa = GetAOA() / RAD;
	slip = GetSlipAngle()/RAD;
	mach = GetMachNumber();
	KEAS = 1.94384 * sqrt(2 * GetDynPressure()) / ATMD; // knot equivalent airspeed
	acc = (tforce.y - weight.y) / (GetMass() * G);
	if (agl < 2500 && elec->RADAR()) {
		//acquire radio speeds
		GetGroundspeedVector(FRAME_LOCAL, gs);
		vx = gs.x / (0.3048 * BetterPB.scale);
		vz = gs.z / (0.3048 * BetterPB.scale);
		GetGroundspeedVector(FRAME_HORIZON, gs);
		vs = 0.2 * (gs.y) / 0.3045;
	}
	else {
		agl = 2501;
		gs = { 0,0,0 };
		vx = 0;
		vz = 0;
		vs = 0;
	}
	//acquire barometric data
	vsfps = (-8.314 * GetAtmTemperature() * log(GetAtmPressure() / ATMP) / (G * 0.029) -alt) / (0.3048*simdt);
	alt = -8.314 * GetAtmTemperature()*log(GetAtmPressure()/ATMP)/(G*0.029);
	if ((vsfps > 500) || (vsfps < -500)) {
		vsfps = 500;
	}
	if ((alt > 0.3048 * 50000) || (alt < 0)) {
		alt = 0;
	}
	if (!BetterPB.ADRext_p && !BetterPB.ADLext_p) {
		vsfps = 0;
		alt = 0;
	}
	if (GetAltitude() > 100000) mach = 0;
	//instrument limits
	if (vx < -4) vx = -4;
	if (vz < -4) vz = -4;
	if (vx > 4) vx = 4;
	if (vz > 4) vz = 4;
	if ((acc < -12) || (acc > 12)) acc = 12;
	if (vs > 6) vs = 6;
	if (vs < -6) vs = -6;
	if (KEAS > 550 || acc > 9 || (agl < 5 && vs < -3)) { BetterPB.ded = true; TriggerPanelRedrawArea(0,AID_DED); }
	if (aoa > 40) { aoa = 40; }
	if (aoa < -40) { aoa = -40; }
	if (mach > 30) { mach = 30; }
	if (volta > 45) { volta = 45; }
	if (volta < 21) { volta = 21; }
	if (voltb > 45) { voltb = 45; }
	if (voltb < 21) { voltb = 21; }
	if (amperea > 40) { amperea = 40; }
	if (ampereb > 40) { ampereb = 40; }
	if (slip > 20) { slip = 20; }
	if (slip < -20) { slip = -20; }
	truc(BetterPB.adimode);
	BetterPB.IMU = elec->IMU();
	if (!BetterPB.IMU) RPY[0] = 0, RPY[1] = 0, RPY[2] = 0;
	//if(BetterPB.rcs == -1) ActivateNavmode(NAVBIT_KILLROT);
}
void ShuttlePB::DefineMainPanel(PANELHANDLE hPanel) // it makes the panel, basically.
{
	static DWORD panelW = 1920;
	static DWORD panelH = 540;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 2048;
	static DWORD texH = 1024;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	static NTVERTEX VTX[4] = {
	{ 0, 0,0, 0,0,0, 0.0f,1.0f - fpanelH / ftexH},
	{ 0,fpanelH,0, 0,0,0, 0.0f,1.0f },
	{fpanelW,fpanelH,0, 0,0,0, fpanelW / ftexW,1.0f },
	{fpanelW, 0,0, 0,0,0, fpanelW / ftexW,1.0f - fpanelH / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	if (hPanelMesh) oapiDeleteMesh(hPanelMesh);
	hPanelMesh = oapiCreateMesh(0, 0); //panel initialization
	MESHGROUP grp = { VTX, IDX, 4, 6, 0, 0, 0, 0, 0 }; //mesh creation
	MESHGROUP ADIgrp = { 0, 0, 0, 0, 0, 1, 0, 0, 0 }; //useful empty mesh
	MESHGROUP Rollgrp = { 0, 0, 0, 0, 0, 2, 0, 0, 0 }; //useful empty mesh also, note the number, which is the texture index of the mesh
	MESHGROUP ggrp = { 0, 0, 0, 0, 0, 2, 0, 0, 0 };
	MESHGROUP VSgrp = { 0, 0, 0, 0, 0, 2, 0, 0, 0 };
	MESHGROUP Barogrp = { 0, 0, 0, 0, 0, 2, 0, 0, 0 };
	MESHGROUP toomuchgrp = { 0, 0, 0, 0, 0, 2, 0, 0, 0 };
	oapiAddMeshGroup(hPanelMesh, &grp); //integrating the mesh into the panel
	thing = oapiAddMeshGroup(hPanelMesh, &ADIgrp);
	notathing = oapiAddMeshGroup(hPanelMesh, &Rollgrp);
	geez = oapiAddMeshGroup(hPanelMesh, &ggrp);
	VSthing = oapiAddMeshGroup(hPanelMesh, &VSgrp);
	barothing = oapiAddMeshGroup(hPanelMesh, &Barogrp);
	//OK, so it's at this moment that we initialize the mesh instruments!
	ADIinit(hPanelMesh, thing);
	Rollinit(hPanelMesh, notathing);
	Ginit(hPanelMesh, geez);
	VSinit(hPanelMesh, VSthing);
	Baroinit(hPanelMesh, barothing);
	Varioinit(hPanelMesh, barothing);
	ADLinit(hPanelMesh, barothing);
	ADRinit(hPanelMesh, barothing);
	fuelel->Init(hPanelMesh, barothing);
	batt1el->Init(hPanelMesh, barothing);
	batt2el->Init(hPanelMesh, barothing);
	MNAVel->Init(hPanelMesh, barothing);
	MNBVel->Init(hPanelMesh, barothing);
	MNAAel->Init(hPanelMesh, barothing);
	MNBAel->Init(hPanelMesh, barothing);
	//End of mesh instruments
	SURFHANDLE paneltex[3] = {panel2dtex,adisurf,instsurf};
	SetPanelBackground(hPanel, paneltex, 3, hPanelMesh, panelW, panelH, 0, PANEL_ATTACH_BOTTOM | PANEL_MOVEOUT_BOTTOM); //panel construction

	// DO NOT USE OAPIREGISTERPANELAREA, IT DOESN'T WORK (4 sum reeson).
	RegisterPanelArea(hPanel, AID_ADI, _R(0, 0, 0, 0), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_KEAS, _R(400, 200, 500, 300), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_GEES, _R(0, 0, 0, 0), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_XPNTR, _R(0, 0, 0, 0), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_AIRDATALEFT, _R(1413, 801-off-10, 1444, 832-off+10), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_AIRDATARIGHT, _R(1490, 801 - off - 10, 1521, 832 - off + 10), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_MAINP, _R(1578, 770-off, 1627, 969-off), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBPRESSED, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_HOVER, _R(1644, 770 - off, 1693, 969 - off), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_LBPRESSED, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_XPNTRSCALE, _R(133, 567 - off, 164, 598 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RADARSWITCH, _R(1355, 895 - off, 1386, 926 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RCS, _R(261,950-off, 292, 981 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_ADIMODE, _R(161, 950 - off, 192, 981 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RADIOSWITCH, _R(1426,895-off, 1457, 926 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_MNASWITCH, _R(1200, 801 - off, 1231, 832 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_MNBSWITCH, _R(1271, 801 - off, 1302, 832 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_IMUSWITCH, _R(1342, 801 - off, 1373, 832 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);

	RegisterPanelArea(hPanel, AID_BAT1CB, _R(1200, 719 - off, 1231, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_BAT2CB, _R(1271, 719 - off, 1302, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_MNACB, _R(1342, 719 - off, 1373, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_MNBCB, _R(1413, 719 - off, 1444, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_IMUMNA, _R(1484, 719 - off, 1515, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_IMUMNB, _R(1555, 719 - off, 1586, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_ADPMNA, _R(1626, 719 - off, 1657, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_ADPMNB, _R(1697, 719 - off, 1728, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RADIOMNA, _R(1768, 719 - off, 1799, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RADIOMNB, _R(1839, 719 - off, 1870, 750 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RDRMNA, _R(1200, 902 - off, 1231, 933 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_RDRMNB, _R(1271, 902 - off, 1302, 933 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_AF, _R(361, 950 - off, 392, 981 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_MAIN, _R(1751, 814 - off, 1782, 845 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	RegisterPanelArea(hPanel, AID_HVR, _R(1822, 814 - off, 1853, 845 - off), PANEL_REDRAW_MOUSE, PANEL_MOUSE_LBDOWN | PANEL_MOUSE_RBDOWN, panel2dtex, NULL);
	//RegisterPanelArea(hPanel, AID_DED, _R(0, 0, 0, 0), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, ded, NULL);
	RegisterPanelArea(hPanel, AID_DED, _R(0, 0, 0, 0), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, panel2dtex, NULL);
}
void ShuttlePB::ScalePanel(PANELHANDLE hPanel, DWORD viewW, DWORD viewH)
{
	double defscale = (double)viewW / 1920;
	double magscale = max(defscale, 1.0);
	SetPanelScaling(hPanel, defscale, magscale);
}
bool ShuttlePB::clbkLoadPanel2D(int id, PANELHANDLE hPanel, DWORD viewW, DWORD viewH)
{
	//It loads panels
	switch (id) {
	case 0:
		DefineMainPanel(hPanel);
		ScalePanel(hPanel, viewW, viewH);
		return true;
	default:
		return false;
	}
}
bool ShuttlePB::clbkPanelMouseEvent(int id, int event, int mx, int my)
{
	// Is the button pressed? Are the thrusers on? That's the question...
	switch (id) {
	case AID_AIRDATALEFT:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.ADLext_p && elec->ADP()) {
			BetterPB.ADLext_p = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.ADLext_p) {
			BetterPB.ADLext_p = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_AIRDATARIGHT:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.ADRext_p && elec->ADP()) {
			BetterPB.ADRext_p = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.ADRext_p) {
			BetterPB.ADRext_p = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_MAINP:
		if (my > 0 && my <= 200) {
			BetterPB.ThrottleM = 200 - my;
			SetThrusterGroupLevel(THGROUP_MAIN, BetterPB.ThrottleM / 200);
		}
		return true;
	case AID_HOVER:
		if (my > 0 && my <= 200) {
			BetterPB.ThrottleH = 200 - my;
			SetThrusterGroupLevel(THGROUP_HOVER, BetterPB.ThrottleH / 200);
		}
		return true;
	case AID_XPNTRSCALE:
		if (event == PANEL_MOUSE_LBDOWN && BetterPB.scale != 25) {
			BetterPB.scale = 25;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.scale != 5) {
			BetterPB.scale = 5;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_RADARSWITCH:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.radar) {
			BetterPB.radar = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.radar) {
			BetterPB.radar = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_RCS:
		if (event==PANEL_MOUSE_LBDOWN && BetterPB.rcs != 1) {
			BetterPB.rcs += 1;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.rcs != -1) {
			BetterPB.rcs -= 1;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (BetterPB.rcs == -1 || BetterPB.rcs == 1) {
			SetAttitudeMode(RCS_ROT);
		} else {
			SetAttitudeMode(RCS_NONE);
		}
		return true;
	case AID_ADIMODE:
		if (event == PANEL_MOUSE_LBDOWN && BetterPB.adimode != 1) {
			BetterPB.adimode += 1;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.adimode != -1) {
			BetterPB.adimode -= 1;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_RADIOSWITCH:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.radio) {
			BetterPB.radio = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.radio) {
			BetterPB.radio = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_MNASWITCH:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.MNA_b1b2) {
			BetterPB.MNA_b1b2 = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.MNA_b1b2) {
			BetterPB.MNA_b1b2 = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_MNBSWITCH:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.MNB_b1b2) {
			BetterPB.MNB_b1b2 = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.MNB_b1b2) {
			BetterPB.MNB_b1b2 = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_IMUSWITCH:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.IMU_p) {
			BetterPB.IMU_p = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.IMU_p) {
			BetterPB.IMU_p = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_MNACB:
		if (!BetterPB.MNA) {
			BetterPB.MNA = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		} else {
			BetterPB.MNA = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_MNBCB:
		if (!BetterPB.MNB) {
			BetterPB.MNB = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.MNB = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_BAT1CB:
		if (!BetterPB.BATT1) {
			BetterPB.BATT1 = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.BATT1 = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_BAT2CB:
		if (!BetterPB.BATT2) {
			BetterPB.BATT2 = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.BATT2 = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_IMUMNA:
		if (!BetterPB.IMU_MNA) {
			BetterPB.IMU_MNA = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.IMU_MNA = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_IMUMNB:
		if (!BetterPB.IMU_MNB) {
			BetterPB.IMU_MNB = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.IMU_MNB = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_ADPMNA:
		if (!BetterPB.ADP_MNA) {
			BetterPB.ADP_MNA = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.ADP_MNA = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_ADPMNB:
		if (!BetterPB.ADP_MNB) {
			BetterPB.ADP_MNB = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.ADP_MNB = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_RADIOMNA:
		if (!BetterPB.RAD_MNA) {
			BetterPB.RAD_MNA = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.RAD_MNA = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_RADIOMNB:
		if (!BetterPB.RAD_MNB) {
			BetterPB.RAD_MNB = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.RAD_MNB = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_RDRMNA:
		if (!BetterPB.RDR_MNA) {
			BetterPB.RDR_MNA = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.RDR_MNA = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_RDRMNB:
		if (!BetterPB.RDR_MNB) {
			BetterPB.RDR_MNB = true;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		else {
			BetterPB.RDR_MNB = false;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\CB.wav");
		}
		return true;
	case AID_AF:
		if (event == PANEL_MOUSE_LBDOWN && BetterPB.af != 1) {
			BetterPB.af += 1;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.af != -1) {
			BetterPB.af -= 1;
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (BetterPB.af == 1) {
			SetADCtrlMode(7);
		} else if (BetterPB.af == 0) {
			SetADCtrlMode(1);
		} else {
			SetADCtrlMode(0);
		}
		return true;
	case AID_MAIN:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.main_arm) {
			BetterPB.main_arm = true;
			SetThrusterResource(th_main, hpr);
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.main_arm) {
			BetterPB.main_arm = false;
			SetThrusterResource(th_main, NULL);
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	case AID_HVR:
		if (event == PANEL_MOUSE_LBDOWN && !BetterPB.hvr_arm) {
			BetterPB.hvr_arm = true;
			SetThrusterResource(th_hover, hpr);
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		if (event == PANEL_MOUSE_RBDOWN && BetterPB.hvr_arm) {
			BetterPB.hvr_arm = false;
			SetThrusterResource(th_hover, NULL);
			LoadAndPlay("Sound\\_CustomVesselsSounds\\BetterPB\\switch.wav");
		}
		return true;
	}
	return false;
}
void ThreeSwitch(SURFHANDLE surf, SURFHANDLE inst,int x, int y, int st) {
	if (st == -1) {
		oapiBlt(surf, inst, x, y, 300, 75, 32, 32);
	}
	else if (st == 1) {
		oapiBlt(surf, inst, x, y, 331, 75, 32, 32);
	}
	else {
		oapiBlt(surf, inst, x, y, 362, 75, 32, 32);
	}
}
void NormalSwitch(SURFHANDLE surf, SURFHANDLE inst, int x, int y, int st) {
	if (!st) {
		oapiBlt(surf, inst, x, y, 300, 75, 32, 32);
	}
	else {
		oapiBlt(surf, inst, x, y, 331, 75, 32, 32);
	}
}
void CaB(SURFHANDLE surf, SURFHANDLE inst, int x, int y, int st) {
	if (st) {
		oapiBlt(surf, inst, x-4, y-4, 300, 150, 40, 40);
	}
	else {
		oapiBlt(surf, inst, x-4, y-4, 340, 150, 40, 40);
	}
}
bool ShuttlePB::clbkPanelRedrawEvent(int id, int event, SURFHANDLE surf)
{
	//Start of redrawing
	switch (id) {
	case AID_ADI:
		ADIredraw(RPY[1], RPY[0]); //This function draws the attitude indicator, so it worth take a look
		return true;
	case AID_GEES:
		//Some instruments
		Gredraw(acc);
		VSredraw(vs);
		Baroredraw(alt / (0.3048 * 10000));
		Varioredraw(vsfps / 100);
		batt1el->Redraw(BetterPB.batt1/ 400);
		fuelel->Redraw(GetFuelMass() / PB_FUELMASS);
		batt2el->Redraw(BetterPB.batt2 / 400);
		MNAVel->Redraw((volta-21)/24);
		MNBVel->Redraw((voltb - 21) / 24);
		MNAAel->Redraw(amperea / 40);
		MNBAel->Redraw(ampereb / 40);
		return true;
	case AID_KEAS:
		//Three digits KEAS
		oapiBlt(surf, instsurf, 522, 200 + off, 0, (int)40.0 + 18.0 * floor(std::fmod(KEAS, 10.0)), 11, 18);
		oapiBlt(surf, instsurf, 511, 200 + off, 0, (int)40.0 + 18.0 * floor(std::fmod(KEAS, 100.0) / 10.0), 11, 18);
		oapiBlt(surf, instsurf, 500, 200 + off, 0, (int)40.0 + 18.0 * floor(std::fmod(KEAS, 1000.0) / 100.0), 11, 18);
		//Three digits ALT
		oapiBlt(surf, instsurf, 522, 719, 0, (int)40.0 + 18.0 * floor(std::fmod(GetAltitude()/1852, 10.0)), 11, 18);
		oapiBlt(surf, instsurf, 511, 719, 0, (int)40.0 + 18.0 * floor(std::fmod(GetAltitude() / 1852, 100.0) / 10.0), 11, 18);
		oapiBlt(surf, instsurf, 500, 719, 0, (int)40.0 + 18.0 * floor(std::fmod(GetAltitude() / 1852, 1000.0) / 100.0), 11, 18);
	case AID_AIRDATALEFT:
		ADLredraw(BetterPB.ADLext_p);
		return true;
	case AID_AIRDATARIGHT:
		ADRredraw(BetterPB.ADRext_p);
		return true;
	case AID_XPNTR:
		oapiBlt(surf, surf, 200, 502, 263, 250, 150, 150);
		//XPNDR needles
		oapiBlt(surf, surf, 200, (int)(576 - 18 * vz), 0, 250, 150, 2);
		oapiBlt(surf, surf, (int)(274 + 18 * vx), 502, 0, 300, 2, 150);
		//Yaw indicator
		oapiBlt(surf, surf, 137, 850, (int)(1 + 2 * std::fmod(RPY[2] / RAD + 300, 360)), 100, 280, 40);
		//Machometer
		oapiBlt(surf, surf, 136, 665, 2007, (int)(1023 - 178 - 20 * mach), 40, 178);
		//RA
		if (agl < 300) {oapiBlt(surf, surf, 378, 665, 1966, (int)(1023 - 179 - agl), 40, 178);}
		else { oapiBlt(surf, surf, 378, 665, 1966, (int)(1023 - 179 - 240- 0.2*agl), 40, 178); }
		//AOA meter
		oapiBlt(surf, surf,717,740, 1925, (int)(445-10*aoa), 40, 178);
		//SLIP meter
		oapiBlt(surf, surf, 770, 930, (int)(442 - 18 * slip),26, 178, 40);
		return true;
	case AID_MAINP:
		oapiBlt(surf, surf, 1578, 760, 1550, 126, 50, 220);
		oapiBlt(surf, surf, 1578, (int)(760+200*(1-GetThrusterGroupLevel(THGROUP_MAIN))), 597, 416, 50, 20);
	return true;
	case AID_HOVER:
		oapiBlt(surf, surf, 1644, 760, 1550, 126, 50, 220);
		oapiBlt(surf, surf, 1644, (int)(760 + 200 * (1 - GetThrusterGroupLevel(THGROUP_HOVER))), 648, 416, 50, 20);
		return true;
	case AID_XPNTRSCALE:
		if(BetterPB.scale == 5) {
			oapiBlt(surf, instsurf, 133, 567,300,75, 32, 32); //down
		}
		else {
			oapiBlt(surf, instsurf, 133, 567, 331, 75, 32, 32); //up
		}
		return true;
	case AID_RADARSWITCH:
		NormalSwitch(surf, instsurf, 1355, 895, BetterPB.radar);
		return true;
	case AID_ADIMODE:
		ThreeSwitch(surf, instsurf, 161, 950, BetterPB.adimode);
		return true;
	case AID_RCS:
		ThreeSwitch(surf, instsurf, 261, 950, BetterPB.rcs);
		return true;
	case AID_RADIOSWITCH:
		NormalSwitch(surf, instsurf, 1426, 895, BetterPB.radio);
		return true;
	case AID_MNASWITCH:
		NormalSwitch(surf, instsurf, 1200, 801, BetterPB.MNA_b1b2);
		return true;
	case AID_MNBSWITCH:
		NormalSwitch(surf, instsurf, 1271, 801, BetterPB.MNB_b1b2);
		return true;
	case AID_IMUSWITCH:
		NormalSwitch(surf, instsurf, 1342, 801, BetterPB.IMU_p);
		return true;
	case AID_MNACB:
		CaB(surf, instsurf, 1342, 719, BetterPB.MNA);
		return true;
	case AID_MNBCB:
		CaB(surf, instsurf, 1413, 719, BetterPB.MNB);
		return true;
	case AID_BAT1CB:
		CaB(surf, instsurf, 1200, 719, BetterPB.BATT1);
		return true;
	case AID_BAT2CB:
		CaB(surf, instsurf, 1271, 719, BetterPB.BATT2);
		return true;
	case AID_IMUMNA:
		CaB(surf, instsurf, 1484, 719, BetterPB.IMU_MNA);
		return true;
	case AID_IMUMNB:
		CaB(surf, instsurf, 1555, 719, BetterPB.IMU_MNB);
		return true;
	case AID_ADPMNA:
		CaB(surf, instsurf, 1626, 719, BetterPB.ADP_MNA);
		return true;
	case AID_ADPMNB:
		CaB(surf, instsurf, 1697, 719, BetterPB.ADP_MNB);
		return true;
	case AID_RADIOMNA:
		CaB(surf, instsurf, 1768, 719, BetterPB.RAD_MNA);
		return true;
	case AID_RADIOMNB:
		CaB(surf, instsurf, 1839, 719, BetterPB.RAD_MNB);
		return true;
	case AID_RDRMNA:
		CaB(surf, instsurf, 1200, 902, BetterPB.RDR_MNA);
		return true;
	case AID_RDRMNB:
		CaB(surf, instsurf, 1271, 902, BetterPB.RDR_MNB);
		return true;
	case AID_AF: 
		ThreeSwitch(surf, instsurf, 361, 950, BetterPB.af);
		return true;
	case AID_MAIN:
		NormalSwitch(surf, instsurf, 1751, 814, BetterPB.main_arm);
		return true;
	case AID_HVR:
		NormalSwitch(surf, instsurf, 1822, 814, BetterPB.hvr_arm);
		return true;
	case AID_DED:
		if (BetterPB.ded) {
			oapi::Font* font = oapiCreateFont(38, true, "Sans", FONT_BOLD);
			oapi::Sketchpad* skp = oapiGetSketchpad(surf);
			if (skp) {
				skp->SetFont(font);
				skp->SetTextColor(0x0000FF);
				skp->SetTextAlign(oapi::Sketchpad::CENTER);
				char cbuf[32];
				strncpy(cbuf, "DEAD !!!!", 10);
				int len = min(strlen("DEAD !!!!"), 10);
				skp->Text(850, 700, cbuf, len);
				oapiReleaseSketchpad(skp);
			}
			oapiReleaseFont(font);
		}
	}
	return false;
}
/*bool ShuttlePB::clbkDrawHUD(int mode, const HUDPAINTSPEC* hps, oapi::Sketchpad* skp)
{

	// Default HUD elements and subsystem HUD drawing
	VESSEL3::clbkDrawHUD(mode, hps, skp);

	int cx = hps->CX, cy = hps->CY;
	int d = hps->Markersize * 5;
	// show RCS mode
	switch (GetAttitudeMode()) {
	case RCS_ROT:
		skp->Text(0, hps->H - 20, "RCS ROT", 7);
		break;
	case RCS_LIN:
		skp->Text(0, hps->H - 20, "RCS_LIN", 7);
		break;
	}

	if (oapiGetHUDMode() == HUD_DOCKING) {
				skp->Line(cx - d, cy - d, cx + d, cy + d);
				skp->Line(cx - d, cy + d, cx + d, cy - d);
	}
			//char* str = "YOU'RE DEAD !!!!";
			//int w = skp->GetTextWidth(str);
			//skp->Text(cx - w / 2, cy, str, 8);
	return true;
}*/
/*
All the overloaded mesh panel elements
this script will never end... HALP!!!
*/
void ShuttlePB::ADIinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelW = 156;
	static DWORD panelH = 140;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 1024;
	static DWORD texH = 1024;
	static DWORD texX = 433;
	static DWORD texY = 441;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ 200, 200,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ 200,200+fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY+fpanelH) / ftexH },
	{200+fpanelW,200+fpanelH,0, 0,0,0, (fpanelW+ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{200+fpanelW, 200,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	adigrp = oapiMeshGroup(hMesh, grpidx); //The mesh group is still empty now.
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
	/*OK, we need to talk about this undocumented function
	I discovered it when I was reading instruments.cpp.
	I don't know exactly what it do, but I think it links the ADI mesh with the background mesh.
	Note that grp is now linked to the ADI mesh, so if we change something in grp, it will change something in the ADI.
	*/
}
void ShuttlePB::Rollinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 278;
	static DWORD panelY = 270;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 40;
	static DWORD panelH = 39;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	static NTVERTEX VTX[4] = { //so complicated
	{fpanelX-fpanelW/2.0, fpanelY - fpanelH / 2.0,0, 0,0,0, 0,0},
	{ fpanelX - fpanelW / 2.0,fpanelY + fpanelH / 2.0,0, 0,0,0, 0,fpanelH / ftexH },
	{fpanelX + fpanelW / 2.0,fpanelY + fpanelH / 2.0,0, 0,0,0, fpanelW / ftexW,fpanelH / ftexH },
	{fpanelX + fpanelW / 2.0, fpanelY - fpanelH / 2.0,0, 0,0,0, fpanelW/ftexW,0} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	grproll = oapiMeshGroup(hMesh, grpidx); //The mesh group is empty now.
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::Ginit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 858-7;
	static DWORD panelY = 828-484-82+7;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 15;
	static DWORD panelH = 82;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 17;
	static DWORD texY = 52;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ panelX, panelY,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ panelX,panelY + fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW,panelY + fpanelH,0, 0,0,0, (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW, panelY,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	geezgrp = oapiMeshGroup(hMesh, grpidx);
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::VSinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 474;
	static DWORD panelY = 877 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 50;
	static DWORD panelH = 3;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 17;
	static DWORD texY = 145;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ panelX, panelY,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ panelX,panelY + fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW,panelY + fpanelH,0, 0,0,0, (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW, panelY,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	vsgrp = oapiMeshGroup(hMesh, grpidx);
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::Baroinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 495;
	static DWORD panelY = 576 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 9;
	static DWORD panelH = 85;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 153;
	static DWORD texY = 74;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ panelX, panelY,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ panelX,panelY + fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW,panelY + fpanelH,0, 0,0,0, (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW, panelY,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	barogrp = oapiMeshGroup(hMesh, grpidx);
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::Varioinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 680;
	static DWORD panelY = 576 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 9;
	static DWORD panelH = 85;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 153;
	static DWORD texY = 74;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ panelX, panelY,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ panelX,panelY + fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW,panelY + fpanelH,0, 0,0,0, (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW, panelY,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	//barogrp = oapiMeshGroup(hMesh, grpidx);
	vtxofs = barogrp->nVtx;
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::ADRinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 1490;
	static DWORD panelY = 801-15 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 32;
	static DWORD panelH = 62;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 72;
	static DWORD texY = 54;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ panelX, panelY,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ panelX,panelY + fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW,panelY + fpanelH,0, 0,0,0, (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW, panelY,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	barogrp = oapiMeshGroup(hMesh, grpidx);
	vtxofs3 = barogrp->nVtx;
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::ADLinit(MESHHANDLE hMesh, DWORD grpidx) {
	static DWORD panelX = 1413;
	static DWORD panelY = 801-15 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 32;
	static DWORD panelH = 62;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 72;
	static DWORD texY = 54;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	static NTVERTEX VTX[4] = { //so complicated
	{ panelX, panelY,0, 0,0,0, ftexX / ftexW,ftexY / ftexH},
	{ panelX,panelY + fpanelH,0, 0,0,0, ftexX / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW,panelY + fpanelH,0, 0,0,0, (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH },
	{panelX + fpanelW, panelY,0, 0,0,0, (fpanelW + ftexX) / ftexW,ftexY / ftexH} };
	static WORD IDX[6] = {
	0,2,1,
	2,0,3
	};
	barogrp = oapiMeshGroup(hMesh, grpidx);
	vtxofs4 = barogrp->nVtx;
	oapiAddMeshGroupBlock(hMesh, grpidx, VTX, 4, IDX, 6);
}
void ShuttlePB::ADRredraw(bool on) {
	static DWORD panelW = 32;
	static DWORD panelH = 62;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 72;
	static DWORD texY = 54;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	if (on) {
		barogrp->Vtx[vtxofs3 + 0].tu = (ftexX - 31) / ftexW;
		barogrp->Vtx[vtxofs3 + 1].tu = (ftexX - 31) / ftexW;
		barogrp->Vtx[vtxofs3 + 2].tu = (fpanelW + ftexX - 31) / ftexW;
		barogrp->Vtx[vtxofs3 + 3].tu = (fpanelW + ftexX - 31) / ftexW;
	}
	else {
		barogrp->Vtx[vtxofs3 + 0].tu = ftexX / ftexW;
		barogrp->Vtx[vtxofs3 + 1].tu = ftexX / ftexW;
		barogrp->Vtx[vtxofs3 + 2].tu = (fpanelW + ftexX) / ftexW;
		barogrp->Vtx[vtxofs3 + 3].tu = (fpanelW + ftexX) / ftexW;
	}
}
void ShuttlePB::ADLredraw(bool on) {
	static DWORD panelW = 32;
	static DWORD panelH = 62;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 512;
	static DWORD texH = 512;
	static DWORD texX = 72;
	static DWORD texY = 54;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	if (on) {
		barogrp->Vtx[vtxofs4 + 0].tu = (ftexX-31) / ftexW;
		barogrp->Vtx[vtxofs4 + 1].tu = (ftexX-31) / ftexW;
		barogrp->Vtx[vtxofs4 + 2].tu = (fpanelW + ftexX-31) / ftexW;
		barogrp->Vtx[vtxofs4 + 3].tu = (fpanelW + ftexX-31) / ftexW;
	}
	else {
		barogrp->Vtx[vtxofs4 + 0].tu = ftexX / ftexW;
		barogrp->Vtx[vtxofs4 + 1].tu = ftexX / ftexW;
		barogrp->Vtx[vtxofs4 + 2].tu = (fpanelW + ftexX) / ftexW;
		barogrp->Vtx[vtxofs4 + 3].tu = (fpanelW + ftexX) / ftexW;
	}
}
void ShuttlePB::Varioredraw(double vario) {
	static DWORD panelX = 680;
	static DWORD panelY = 576 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 9;
	static DWORD panelH = 85;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	float angl = RAD * 36 * vario + PI05;
	float v1[2] = { fpanelX, fpanelY };
	float v2[2] = { fpanelX, fpanelY + fpanelH };
	float v3[2] = { fpanelX + fpanelW, fpanelY + fpanelH };
	float v4[2] = { fpanelX + fpanelW, fpanelY };
	float ofs[2] = { 684,580 - 484 };
	barogrp->Vtx[vtxofs+0].x = Madrixx(angl, v1, ofs);
	barogrp->Vtx[vtxofs + 1].x = Madrixx(angl, v2, ofs);
	barogrp->Vtx[vtxofs + 2].x = Madrixx(angl, v3, ofs);
	barogrp->Vtx[vtxofs + 3].x = Madrixx(angl, v4, ofs);
	barogrp->Vtx[vtxofs + 0].y = Madrixy(angl, v1, ofs);
	barogrp->Vtx[vtxofs + 1].y = Madrixy(angl, v2, ofs);
	barogrp->Vtx[vtxofs + 2].y = Madrixy(angl, v3, ofs);
	barogrp->Vtx[vtxofs + 3].y = Madrixy(angl, v4, ofs);
}
void ShuttlePB::Baroredraw(double balt) {
	static DWORD panelX = 495;
	static DWORD panelY = 576 - off;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 9;
	static DWORD panelH = 85;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	float angl = RAD * 72 * balt;
	float v1[2] = { fpanelX, fpanelY };
	float v2[2] = { fpanelX, fpanelY + fpanelH };
	float v3[2] = { fpanelX + fpanelW, fpanelY + fpanelH };
	float v4[2] = { fpanelX + fpanelW, fpanelY };
	float ofs[2] = { 499,580 - 484 };
	barogrp->Vtx[0].x = Madrixx(angl, v1, ofs);
	barogrp->Vtx[1].x = Madrixx(angl, v2, ofs);
	barogrp->Vtx[2].x = Madrixx(angl, v3, ofs);
	barogrp->Vtx[3].x = Madrixx(angl, v4, ofs);
	barogrp->Vtx[0].y = Madrixy(angl, v1, ofs);
	barogrp->Vtx[1].y = Madrixy(angl, v2, ofs);
	barogrp->Vtx[2].y = Madrixy(angl, v3, ofs);
	barogrp->Vtx[3].y = Madrixy(angl, v4, ofs);
}
void ShuttlePB::VSredraw(double vs) {
	static DWORD panelX = 474;
	static DWORD panelY = 877 - 484;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 50;
	static DWORD panelH = 3;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	float angl = RAD*15*vs;
	float v1[2] = { fpanelX, fpanelY };
	float v2[2] = { fpanelX, fpanelY + fpanelH };
	float v3[2] = { fpanelX + fpanelW, fpanelY + fpanelH };
	float v4[2] = { fpanelX + fpanelW, fpanelY };
	float ofs[2] = { 540,878-484 };
	vsgrp->Vtx[0].x = Madrixx(angl, v1, ofs);
	vsgrp->Vtx[1].x = Madrixx(angl, v2, ofs);
	vsgrp->Vtx[2].x = Madrixx(angl, v3, ofs);
	vsgrp->Vtx[3].x = Madrixx(angl, v4, ofs);
	vsgrp->Vtx[0].y = Madrixy(angl, v1, ofs);
	vsgrp->Vtx[1].y = Madrixy(angl, v2, ofs);
	vsgrp->Vtx[2].y = Madrixy(angl, v3, ofs);
	vsgrp->Vtx[3].y = Madrixy(angl, v4, ofs);
}
void ShuttlePB::Gredraw(double gz) {
	static DWORD panelX = 858 - 7;
	static DWORD panelY = 828 - 484 - 82 + 7;
	float fpanelX = (float)panelX;
	float fpanelY = (float)panelY;
	static DWORD panelW = 15;
	static DWORD panelH = 82;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	float angl = -PI05*(1-gz/6);
	float v1[2] = {fpanelX, fpanelY};
	float v2[2] = { fpanelX, fpanelY + fpanelH };
	float v3[2] = { fpanelX + fpanelW, fpanelY + fpanelH };
	float v4[2] = { fpanelX + fpanelW, fpanelY };
	float ofs[2] = {858,828-484};
	geezgrp->Vtx[0].x = Madrixx(angl,v1,ofs);
	geezgrp->Vtx[1].x = Madrixx(angl, v2, ofs);
	geezgrp->Vtx[2].x = Madrixx(angl, v3, ofs);
	geezgrp->Vtx[3].x = Madrixx(angl, v4, ofs);
	geezgrp->Vtx[0].y = Madrixy(angl, v1, ofs);
	geezgrp->Vtx[1].y = Madrixy(angl, v2, ofs);
	geezgrp->Vtx[2].y = Madrixy(angl, v3, ofs);
	geezgrp->Vtx[3].y = Madrixy(angl, v4, ofs);
}
void ShuttlePB::ADIredraw(double pit, double roll) {
	static DWORD panelW = 156;
	static DWORD panelH = 140;
	float fpanelW = (float)panelW;
	float fpanelH = (float)panelH;
	static DWORD texW = 1024;
	static DWORD texH = 1024;
	static DWORD texX = 433;
	static DWORD texY = 441;
	float ftexW = (float)texW;
	float ftexH = (float)texH;
	float ftexX = (float)texX;
	float ftexY = (float)texY;
	float fpit = (float)(180*pit/PI);
	float froll = (float)-roll;
	float v1[2] = { ftexX / ftexW,ftexY / ftexH - 3.5 * fpit / ftexH };
	float v2[2] = { ftexX / ftexW,(ftexY + fpanelH) / ftexH - 3.5 * fpit / ftexH };
	float v3[2] = { (fpanelW + ftexX) / ftexW,(ftexY + fpanelH) / ftexH - 3.5 * fpit / ftexH };
	float v4[2] = { (fpanelW + ftexX) / ftexW,ftexY / ftexH - 3.5 * fpit / ftexH };
	float ofs[2]= { (fpanelW/2 + ftexX) / ftexW,(ftexY + fpanelH/2) / ftexH - 3.5 * fpit / ftexH };
	adigrp->Vtx[0].tv = Madrixy(froll, v1, ofs);
	adigrp->Vtx[0 + 1].tv = Madrixy(froll, v2, ofs);
	adigrp->Vtx[0 + 2].tv = Madrixy(froll, v3, ofs);
	adigrp->Vtx[0 + 3].tv = Madrixy(froll, v4, ofs);
	adigrp->Vtx[0].tu = Madrixx(froll, v1, ofs);
	adigrp->Vtx[0 + 1].tu = Madrixx(froll, v2, ofs);
	adigrp->Vtx[0 + 2].tu = Madrixx(froll, v3, ofs);
	adigrp->Vtx[0 + 3].tu = Madrixx(froll, v4, ofs);
	/*Little explanation here
	The meshgroup of the attitude indicator (grp) has a list of vertex, called Vtx.
	One element of Vtx, like Vtx[1], have a variable in, called tv, which represent the y axis in the texture surface adisurf
	What we do is basically add the pitch value to tv.
	*/
}
//void Deddraw() {}
// ==============================================================
// Airfoil lift/drag functions
// ==============================================================

void ShuttlePB::vlift (VESSEL *v, double aoa, double M, double Re,
	void *context, double *cl, double *cm, double *cd)
{
	static const double clp[] = {  // lift coefficient from -pi to pi in 10deg steps
		-0.1,-0.5,-0.4,-0.1,0,0,0,0,0,0,0,0,0,0,-0.2,-0.6,-0.6,-0.4,0.2,0.5,0.9,0.8,0.2,0,0,0,0,0,0,0,0,0,0.1,0.4,0.5,0.3,-0.1,-0.5
	};
	static const double aoa_step = 10.0*RAD;
	double a, fidx, saoa = sin(aoa);
	a = modf((aoa+PI)/aoa_step, &fidx);
	int idx = (int)(fidx+0.5);
	*cl = clp[idx]*(1.0-a) + clp[idx+1]*a;     // linear interpolation
	*cm = 0.0; //-0.03*sin(aoa-0.1);
	*cd = 0.03 + 0.4*saoa*saoa;                // profile drag
	*cd += oapiGetInducedDrag (*cl, 1.0, 0.5); // induced drag
	*cd += oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);  // wave drag
}

void ShuttlePB::hlift (VESSEL *v, double aoa, double M, double Re,
	void *context, double *cl, double *cm, double *cd)
{
	static const double clp[] = {  // lift coefficient from -pi to pi in 45deg steps
		0,0.4,0,-0.4,0,0.4,0,-0.4,0,0.4
	};
	static const double aoa_step = 45.0*RAD;
	double a, fidx;
	a = modf((aoa+PI)/aoa_step, &fidx);
	int idx = (int)(fidx+0.5);
	*cl = clp[idx]*(1.0-a) + clp[idx+1]*a;     // linear interpolation
	*cm = 0.0;
	*cd = 0.03;
	*cd += oapiGetInducedDrag (*cl, 1.5, 0.6); // induced drag
	*cd += oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);  // wave drag
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK void InitModule(HINSTANCE hModule)
{
	hDll = hModule;
	//ShuttlePB::panel2dtex = oapiCreateSurface(LoadBitmap(hDll, MAKEINTRESOURCE(102)),false);
	ShuttlePB::panel2dtex = oapiLoadTexture("BetterPB//Panel.dds");
	ShuttlePB::adisurf = oapiLoadTexture("BetterPB//AH.dds");
	ShuttlePB::instsurf = oapiLoadTexture("BetterPB//Texture.dds");
	//ShuttlePB::ded = oapiLoadTexture("Basicmodule//Ded.dds");
}
// module cleanup
DLLCLBK void ExitModule(HINSTANCE hModule)
{
	oapiDestroySurface(ShuttlePB::panel2dtex);
	//oapiDestroySurface(ShuttlePB::ded);
	oapiDestroySurface(ShuttlePB::adisurf);
	oapiDestroySurface(ShuttlePB::instsurf);
}
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
